%% PANO(PAtch-based Nolocal Operator)-based compressed sensing MRI
%% Xiaobo Qu
%% quxiaobo@xmu.edu.cn
%% http://www.quxiaobo.org
%% Computational Imaging Group(CIG) at Xiamen University
%% Department of Electronic Science, Xiamen University
%% November 2, 2013
%% If you use this code, please cite our papers:
%% [1] Xiaobo Qu, Yingkun Hou, Fan Lam, Di Guo, Jianhui Zhong, Zhong Chen. Magnetic resonance image reconstruction from undersampled measurements using a patch-based nonlocal operator, Medical Image Analysis, in press, http://dx.doi.org/10.1016/j.media.2013.09.007, 2013.
%% All rights are reserved by Xiaobo Qu.
clc;
clear all;
close all;
clear classes;
%% Set the path
Path_PANO_CS_MRI
%% Load data
load T2wBrain_slice_10
load mask_040
%% Show the sampling pattern
figure,imshow(mask)
[row,column]=size(mask);
SamplingRate=sum(mask(:))/numel(mask);
%% Undersampling the k-space (Fourier coefficients)
imwrite(abs(im_ori),'im_ori.png')
kspace_full=fft2(im_ori);
U=U_Operator(ifftshift(mask),row,column);
y=U*kspace_full;
%% Guide image: zero-filling image
im_rec_zerofilling=ifft2(U'*y);
figure,imshow(abs([im_ori im_rec_zerofilling]),[0 1]),colorbar
imwrite(abs(im_rec_zerofilling),'im_rec_zerofilling.png')
%% PANO-based CS-MRI reconstruction (guide image from undersampled data)
disp('-----------------------------------')
paramsPANO.lamda=1e6; % regularization parameter
paramsPANO.im_ref_init=im_rec_zerofilling; % initial guide image
paramsPANO.iter_optBasis=2; % times to learn the nonlocal similarity
tic
[im_rec_PANO_Fast]=solver_PANO_Fast_CS_MRI(y,ifftshift(mask),paramsPANO);
CPU_time_PANO_Fast=toc
RLNE_PANO_Fast=RLNE(im_ori,im_rec_PANO_Fast)
imwrite(abs(im_rec_PANO_Fast),'im_rec_PANO_Fast_Ref_Zerofilling.png')
disp('-----------------------------------')
%% PANO-based CS-MRI reconstruction (guide image from fully sampled data)
disp('-----------------------------------')
paramsPANO.lamda=1e6; % regularization parameter
paramsPANO.im_ref_init=im_ori; % initial guide image
paramsPANO.iter_optBasis=1; % times to learn the nonlocal similarity
tic
[im_rec_PANO_Fast_Ori]=solver_PANO_Fast_CS_MRI(y,ifftshift(mask),paramsPANO);
CPU_time_PANO_Fast_Ori=toc
RLNE_PANO_Fast_Ori=RLNE(im_ori,im_rec_PANO_Fast_Ori)
imwrite(abs(im_rec_PANO_Fast_Ori),'im_rec_PANO_Fast_Ref_Ori.png')
disp('-----------------------------------')
%%
save proj_PANO